// Copyright (c) 2023 Tudor Stanciu

import { useContext } from "react";
import { TuitioContext, TuitioDispatchContext } from "../contexts";
import { TuitioClient } from "@flare/tuitio-client";
import { TuitioLoginResponse, TuitioLogoutResponse } from "@flare/tuitio-client";

export type TuitioClientHookOptions = {
  onLoginSuccess?: (userName: string, token: string, expiresIn: number, validUntil: Date) => void;
  onLoginFailed?: (error: string) => void;
  onLoginError?: (error: any) => void;
  onLogoutSuccess?: (userName: string, logoutDate: Date) => void;
  onLogoutFailed?: (error: string) => void;
  onLogoutError?: (error: any) => void;
};

const defaultOptions: TuitioClientHookOptions = {
  onLoginSuccess: undefined,
  onLoginFailed: undefined,
  onLoginError: undefined,
  onLogoutSuccess: undefined,
  onLogoutFailed: undefined,
  onLogoutError: undefined
};

const useTuitioClient = (options: TuitioClientHookOptions = defaultOptions) => {
  const state = useContext(TuitioContext);
  const dispatchActions = useContext(TuitioDispatchContext);
  const { onLoginSuccess, onLoginFailed, onLoginError, onLogoutSuccess, onLogoutFailed, onLogoutError } = options;

  const login = async (userName: string, password: string): Promise<TuitioLoginResponse> => {
    try {
      const client = new TuitioClient(state.configuration.tuitioUrl);
      const response = await client.login(userName, password);
      if (!response.error) {
        if (!response.result) throw new Error("Unexpected error: Login result and error are both null.");
        dispatchActions.onLoginSuccess(response.result.token, response.result.validUntil, userName);
        onLoginSuccess &&
          onLoginSuccess(userName, response.result.token, response.result.expiresIn, response.result.validUntil);
      } else {
        onLoginFailed && onLoginFailed(response.error);
      }
      return response;
    } catch (error) {
      onLoginError && onLoginError(error);
      throw error;
    }
  };

  const logout = async (): Promise<TuitioLogoutResponse> => {
    try {
      const client = new TuitioClient(state.configuration.tuitioUrl);
      const response = await client.logout();
      if (!response.error) {
        if (!response.result) throw new Error("Unexpected error: Logout result and error are both null.");
        dispatchActions.onLogoutSuccess();
        onLogoutSuccess && onLogoutSuccess(response.result.userName, response.result.logoutDate);
      } else {
        onLogoutFailed && onLogoutFailed(response.error);
      }
      return response;
    } catch (error: any) {
      onLogoutError && onLogoutError(error);
      throw error;
    }
  };

  return { login, logout };
};

export { useTuitioClient };
